# -*- coding: utf-8 -*-
#
#  ExternalMovieInfoLoader
#
#  Coded by dre (c) 2014 - 2015
#  Coding idea and design by dre
#  E-Mail: dre@dreambox-tools.info
#  Support: board.dreambox-tools.info
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Multimedia GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Multimedia GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Multimedia GmbH.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.
# In case of any modification of the code or parts of it you MUST request your own API-key.

# TODO:
# - ContextMenu-Version entfernen
# - Uebersetzungen
# - Code vereinfachen
# - Cover vorhanden / Descr nicht vorhanden: was tun? Abfrage auf mode

from Components.config import config, ConfigSubsection, ConfigSelection, getConfigListEntry, ConfigYesNo
from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaBlend
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Plugins.Plugin import PluginDescriptor
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.Directories import fileExists, SCOPE_CURRENT_SKIN, resolveFilename
from Tools.LoadPixmap import LoadPixmap
from Tools.NumericalTextInput import NumericalTextInput

from enigma import eServiceCenter, eServiceReference, eEnv
from os import listdir as os_listdir, path as os_path, system as os_system
from re import compile
from twisted.web.client import downloadPage, getPage
from urllib import quote
import json

from Components.UsageConfig import defaultMoviePath

if fileExists("/usr/lib/enigma2/python/Plugins/SystemPlugins/Toolkit/NTIVirtualKeyBoard.pyo"):
	from Plugins.SystemPlugins.Toolkit.NTIVirtualKeyBoard import NTIVirtualKeyBoard
	ntikb = True
else:
	from Screens.VirtualKeyBoard import VirtualKeyBoard
	ntikb = False

# for localized messages
from . import _

BASEURL = 'http://api.themoviedb.org/3/'
APIKEY = '55fb261f7fea3666392f98e22b1127d3'

choiceList = [(eEnv.resolve("${datadir}/enigma2/cover"))]
                
dirList = os_listdir("/media")
blockList =  ["hdd","cf","sdcard","usb"]

if fileExists("/proc/mounts"):
	mountsFile = open("/proc/mounts" ,"r")
	for line in mountsFile:
		entry = line.split()
		if entry[2] in ["nfs", "nfs4", "smbfs", "cifs"]:
			if entry[1].startswith("/media/"):
				print "[EMIL] - found network drive. Exclude ", entry[1]
				blockList.append(entry[1][7:])
	mountsFile.close()

	for dir in dirList:
		if dir in blockList:
			continue
		if os_path.ismount("/media/%s" %(dir)) or (os_path.islink("/media/%s" %(dir)) and os_path.ismount(os_path.realpath("/media/%s" %(dir)))):
			choiceList.extend([("/media/%s/cover" %(dir))])
	if os_path.ismount("/media/cf") or (os_path.islink("/media/cf") and os_path.ismount(os_path.realpath("/media/cf"))):
		choiceList.extend([("/media/cf/cover")])
	if os_path.ismount("/media/usb") or (os_path.islink("/media/usb") and os_path.ismount(os_path.realpath("/media/usb"))):
		choiceList.extend([("/media/usb/cover")])
	if os_path.ismount("/media/sdcard") or (os_path.islink("/media/sdcard") and os_path.ismount(os_path.realpath("/media/sdcard"))):
		choiceList.extend([("/media/sdcard/cover")])                   
	if os_path.ismount("/data"):
		choiceList.extend([("/data/cover")])
	if os_path.ismount("/dreambox-data"):
		choiceList.extend([("/dreambox-data/cover")])

# config			
config.plugins.externalmovieinfoloader = ConfigSubsection()
config.plugins.externalmovieinfoloader.coverpath = ConfigSelection(choiceList, default = eEnv.resolve("${datadir}/enigma2/cover"))
config.plugins.externalmovieinfoloader.askfortype = ConfigYesNo(default=False)
config.plugins.externalmovieinfoloader.usefirstentry = ConfigYesNo(default=False)
config.plugins.externalmovieinfoloader.infolanguage = ConfigSelection([("en", _("English")), ("de", _("German"))], default="en")
config.plugins.externalmovieinfoloader.askforreplace = ConfigYesNo(default=False)
config.plugins.externalmovieinfoloader.mode = ConfigSelection(choices = [("both", _("Cover & Description")),("cover",_("Cover")),("description",_("Description"))], default = "both")
config.plugins.externalmovieinfoloader.searchlanguage = ConfigSelection([("en", _("English")), ("de", _("German"))], default="en")


#pattern: show.SnnEnn.filetype / show - SnnEnn - episode.filetype
tvseries = compile('(.*?)[\s\.-]*[S|s][0-9]+[E|e][0-9](.*?)')
#pattern: SnnEnn.show.filetype / SnnEnn - show.filetype
tvseriesalt = compile('^[S|s][0-9]+[E|e][0-9]+[\s\.\-]*(.*?)\.[ts|m2ts|mp3|wav|ogg|jpg|jpeg|jpe|png|bmp|mpg|mpeg|mkv|mp4|mov|divx|avi|mp2|m4a|flac|ifo|vob|iso|sh|flv|3gp|mod|wmv]')
#pattern: show - season (episodenr/total episodes) (no filetype!)
tvseriestotal= compile('(.*?)[\s-][\s](.*?)\([0-9]+\/[0-9]+\)') #.[ts|m2ts|mp3|wav|ogg|jpg|jpeg|jpe|png|bmp|mpg|mpeg|mkv|mp4|mov|divx|avi|mp2|m4a|flac|ifo|vob|iso|sh|flv|3gp|mod|wmv]')
#pattern: movies
movietitle = compile('(.*\w)\.[ts|m2ts|mp3|wav|ogg|jpg|jpeg|jpe|png|bmp|mpg|mpeg|mkv|mp4|mov|divx|avi|mp2|m4a|flac|ifo|vob|iso|sh|flv|3gp|mod|wmv]')

class ExternalMovieInfoLoaderSearchResult(Screen, HelpableScreen):
	skin = """
		<screen name="ExternalMovieInfoLoaderSearchResult" position="center,center" size="720,550" title="ExternalMovieInfoLoader - Search Result">
			<widget name="SearchString" font="Regular;18" position="0,0" size="720,20" transparent="1" halign="center" />
			<ePixmap alphatest="on" pixmap="skin_default/progress_small.png" position="80,30" size="560,3"/>
			<widget source="list" render="Listbox" position="0,40" size="720,375" scrollbarMode="showOnDemand">
				<convert type="TemplatedMultiContent">
					{"template": [
									MultiContentEntryText(pos=(0,0), size=(490,20), flags=RT_HALIGN_LEFT, text=0),
									MultiContentEntryText(pos=(490,0), size=(100,20), flags=RT_HALIGN_RIGHT, text=1),
									MultiContentEntryText(pos=(590,0), size=(100,20), flags=RT_HALIGN_RIGHT, text=6),
					],
					"fonts": [gFont("Regular",18)],
					"itemHeight": 25
					}
				</convert>
			</widget>
			<widget name="Description" position="30,440" size="570,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="Yellow" pixmap="skin_default/buttons/button_yellow.png" position="10,492" zPosition="10" size="16,16" transparent="1" alphatest="on" />			
			<widget name="Search" position="30,490" size="300,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="Red" pixmap="skin_default/buttons/button_red.png" position="10,522" zPosition="10" size="16,16" transparent="1" alphatest="on" />	
			<widget name="Abort" position="30,520" size="300,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<!--widget name="Green" pixmap="skin_default/buttons/button_green.png" position="360,492" zPosition="10" size="16,16" transparent="1" alphatest="on" /-->			
			<!--widget name="MovieData" position="380,490" size="245,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" /-->			
			<widget name="tmdblogo" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ExternalMovieInfoLoader/tmdb.png" position="630,500" size="80,54" transparent="1" alphatest="blend" />
		</screen>
		"""

	def __init__(self,session, menulist, searchString="", folderSearch=False, descriptionOnly=False):
		self.skin = ExternalMovieInfoLoaderSearchResult.skin	
		self.menulist = menulist
		self.searchString = searchString
		self.folderSearch = folderSearch
		self.descriptionOnly = descriptionOnly
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.initiateDownload, _("Start download")),
			"cancel":	(self.close, _("Close")),
		}, -1)		

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"yellow":	(self.editSearchString,_("Edit movie title")),
			"red":		(self.forcedExit, _("Abort search")),
			#"green":	(self.getDescription, _("Download movie data")),
		}, -1)		

		self["list"] = List(self.menulist)
		self["Description"] = Label(_("Press OK to download data for the selected movie"))
		self["Search"] = Label(_("Edit movie title"))
		self["Abort"] = Label(_("Abort search"))
		#self["MovieData"] = Label(_("Download movie data"))
		self["Yellow"] = Pixmap()
		self["Red"] = Pixmap()
		self["Green"] = Pixmap()
		self["tmdblogo"] = Pixmap()
		self["SearchString"] = Label(self.searchString)
		
		self.onShown.append(self.setWindowTitle)
		
		if self.folderSearch and config.plugins.externalmovieinfoloader.usefirstentry.value:
			self.downloadCover()
			
	def getDescription(self):
		self.selectedIndex = self["list"].index
		headers = {
			'Accept':'application/json',
		}
		
		getPage('%smovie/%s?api_key=%s&language=%s' %(BASEURL, self.menulist[self.selectedIndex][3], APIKEY, config.plugins.externalmovieinfoloader.infolanguage.value), method='GET', headers=headers).addCallback(self.writeMovieData).addErrback(self.getError)	
		
	def writeMovieData(self, result):
		info_dict = {}
		#convert result string to a dictionary
		info_dict = json.loads(result)

		os_system("touch '%s.extinfo'" %(self.menulist[self.selectedIndex][7]))
		with open("%s.extinfo" %(self.menulist[self.selectedIndex][7]),'w') as jfile:
			#write dictionary to a file (this must be a dictionary otherwise we will get a string when reading the file
			json.dump(info_dict, jfile)
		
		txt = _("Cover and movie data were downloaded successfully")
		if config.plugins.externalmovieinfoloader.mode.value == "description":
			txt = _("Movie data was downloaded successfully")
		
		if not self.folderSearch:	
			self.session.open(MessageBox,txt, MessageBox.TYPE_INFO, timeout=5, title=_("External Movie Info Loader"))	
		self.close()	
		
	def forcedExit(self):
		self.close("exitDialog")
		
	def editSearchString(self):
		if ntikb:
			self.session.openWithCallback(self.initiateSearch, NTIVirtualKeyBoard, title=_("Search for"), text="%s" %(self.searchString))
		else:
			self.session.openWithCallback(self.initiateSearch, VirtualKeyBoard, title=_("Search for"), text="%s" %(self.searchString))
		
	def initiateSearch(self, searchString):	
		if searchString is None:
			searchString = ""
		self.close(searchString)
		
	def setWindowTitle(self):
		self.setTitle(_("ExternalMovieInfoLoader - Search Result"))
	
	def initiateDownload(self):
		if config.plugins.externalmovieinfoloader.mode.value == "description" or self.descriptionOnly:
			self.getDescription()
		else:
			self.downloadCover()
	
	def downloadCover(self):
		#self.close(self["list"].index)
		
		self.filename = ""
		if len(self.menulist) > 0:
			if self.folderSearch and config.plugins.externalmovieinfoloader.usefirstentry.value:
				selectedIndex = 0
			else:
				selectedIndex = self["list"].index
	
			url = self.menulist[selectedIndex][4]
			path = config.plugins.externalmovieinfoloader.coverpath.value
			self.filename = self.menulist[selectedIndex][5]

			if not fileExists(path):
				os_system("mkdir %s" %(path))

			downloadPage(url, file("%s/%s.jpg" %(path, self.filename),'w')).addErrback(self.getError).addCallback(self.downloadFinished)
		
	def downloadFinished(self, result):
		print "[EMIL] - finished download of %s" %(self.filename)
		
		if config.plugins.externalmovieinfoloader.mode.value == "cover":
			if not self.folderSearch:
				self.session.open(MessageBox,_("The cover %s was downloaded successfully") %(self.filename), MessageBox.TYPE_INFO, timeout=5, title=_("External Movie Info Loader"))
			self.close()
		else:
			self.getDescription()
		
	def getError(self, error):
		print error

class ExternalMovieInfoLoaderSetup(Screen, HelpableScreen, ConfigListScreen):
	skin = """
		<screen name="ExternalMovieInfoLoaderSetup" position="center,center" size="750,300" title="ExternalMovieInfoLoader Setup" >
			<widget name="config" position="0,0" size="750,200" scrollbarMode="showOnDemand" />
			<ePixmap alphatest="on" pixmap="skin_default/buttons/button_green.png" position="10,222" size="16,16"/>
			<widget name="SaveText" font="Regular;18" position="40,220" size="460,20" transparent="1"/>
			<widget name="Credits" font="Regular;18" position="10,250" size="640,40" transparent="1"/>
		</screen>"""
		
	def __init__(self, session, args = 0):
		self.skin = ExternalMovieInfoLoaderSetup.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.list = []
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.close, [_("Close"),"Close the setup"]),
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveConfig, _("Save config")),
		}, -1)
		
	
		self["SaveText"] = Label(_("Save config and close"))
		self["Credits"] = Label("This product uses the TMDb API but is not endorsed or certified by TMDb.")
		
		ConfigListScreen.__init__(self, self.list)
		
		self.list.append(getConfigListEntry(_("Cover path:"), config.plugins.externalmovieinfoloader.coverpath))
		#self.list.append(getConfigListEntry(_("Ask for type in single mode:"), config.plugins.externalmovieinfoloader.askfortype))
		self.list.append(getConfigListEntry(_("Ask whether to replace existing cover in single mode:"), config.plugins.externalmovieinfoloader.askforreplace))
		self.list.append(getConfigListEntry(_("Always use first search result in multi mode:"), config.plugins.externalmovieinfoloader.usefirstentry))
		self.list.append(getConfigListEntry(_("Language for tmdb data:"), config.plugins.externalmovieinfoloader.infolanguage))
		self.list.append(getConfigListEntry(_("Language for search:"), config.plugins.externalmovieinfoloader.searchlanguage))
		self.list.append(getConfigListEntry(_("Download:"), config.plugins.externalmovieinfoloader.mode))
		self["config"].setList(self.list)
		
	def saveConfig(self):
		for x in self.list:
			x[1].save()
		self.close()

class ExternalMovieInfoLoader(Screen, HelpableScreen):
	skin = """
			<screen name="ExternalMovieInfoLoader" position="center,center" size="900,630" title="External Movie Info Loader">
			<widget source="list" render="Listbox" position="0,0" size="900,550" scrollbarMode="showOnDemand">
				<convert type="TemplatedMultiContent">
					{"template": [
									MultiContentEntryText(pos=(10,0), size=(870,20), flags=RT_HALIGN_LEFT, text=2)
					],
					"fonts": [gFont("Regular",18)],
					"itemHeight": 50
					}
				</convert>
			</widget>
			<widget name="ButtonYellowText" position="30,570" size="350,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonYellow" pixmap="skin_default/buttons/button_yellow.png" position="10,570" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonBlueText" position="470,570" size="350,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonBlue" pixmap="skin_default/buttons/button_blue.png" position="450,570" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="tmdblogo" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/ExternalMovieInfoLoader/tmdb.png" position="820,570" size="80,54" transparent="1" alphatest="blend" />
			<widget name="Credits" font="Regular;18" position="10,610" size="780,20" transparent="1"/>
		</screen>
		<!-- for skinners: -->
		<!-- 0 = serviceref -->
		<!-- 1 = name -->
		<!-- 2 = name used for search -->
		<!-- 3 = filename used for cover -->
		<!-- 4 = name of file -->
		<!-- 5 = pointer to pixmap (use to display cover with MultiContentEntryPixmapAlphaBlend) -->
		<!-- 5   example: MultiContentEntryPixmapAlphaBlend(pos=(10,0), size=(100,60), png=5) -->
		"""

	def __init__(self, session, serviceref=None, parent=None):
		self.skin = ExternalMovieInfoLoader.skin
		self.session = session
		self.parent = parent
		Screen.__init__(self, session, parent=parent)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"ok":		(self.getMovieDataForSelected, _("Load movie data for selected movie")),
			"cancel":	(self.exitDialog, _("Close")),
		}, -1)
		
		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
		{
			"menu":	(self.openConfig, _("Open config")),
		}, -1)

		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"blue": (self.getMovieDataForAll, _("Load movie data for all movies")),
			"yellow": (self.getMovieDataForSelected, _("Load movie data for selected movie")),
		}, -1)	
		
		self["NumberActions"] = NumberActionMap(["NumberActions","InputAsciiActions"],
		{
				"gotAsciiCode": self.keyAsciiCode,
				"0": self.keyNumberGlobal,
				"1": self.keyNumberGlobal,
				"2": self.keyNumberGlobal,
				"3": self.keyNumberGlobal,
				"4": self.keyNumberGlobal,
				"5": self.keyNumberGlobal,
				"6": self.keyNumberGlobal,
				"7": self.keyNumberGlobal,
				"8": self.keyNumberGlobal,
				"9": self.keyNumberGlobal,
		})
		
		self["ButtonBlue"] = Pixmap()
		self["ButtonBlueText"] = Label(_("Load movie data for all movies"))
		
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowText"] = Label(_("Load movie data for selected movie"))

		self["tmdblogo"] = Pixmap()
		self["Credits"] = Label("This product uses the TMDb API but is not endorsed or certified by TMDb.")

		self["list"] = List()
		
		self.tmdbconfig = False
		self.folderSearch = False
		self.descriptionOnly = False
		self.movietype = "multi"

		self.numericalTextInput = NumericalTextInput()
		self.numericalTextInput.setUseableChars(u'1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ')
		
		if serviceref is None:
			self.getMovies()
		else:
			self.getMovie(serviceref)

	def keyAsciiCode(self):
		unichar = unichr(getPrevAsciiCode())
		charstr = unichar.encode("utf-8")
		if len(charstr) == 1:
			index = self.getFirstMatchingEntry(charstr[0])
			if index is not None:
				self["list"].setIndex(index)

	def keyNumberGlobal(self, number):
		unichar = self.numericalTextInput.getKey(number)
		charstr = unichar.encode("utf-8")
		if len(charstr) == 1:
			index = self.getFirstMatchingEntry(charstr[0])
			if index is not None:
				self["list"].setIndex(index)
			
	def getFirstMatchingEntry(self, char):
		for i in range(len(self.movieList)):
			if self.movieList[i][2].startswith(char):
				return i

	def openConfig(self):
		self.session.open(ExternalMovieInfoLoaderSetup)

	# used in movielist to get a single movie
	def getMovie(self, serviceref):
		self.serviceHandler = eServiceCenter.getInstance()

		self.movieList = []
		
		if serviceref.valid():
			info = self.serviceHandler.info(serviceref)
			if info:
				name = info.getName(serviceref)
			if tvseriestotal.match(name) is not None:
				searchString = tvseriestotal.match(name).groups()[0]
			elif tvseriesalt.match(name) is not None:
				searchString = tvseriesalt.match(name).groups()[0]
			elif tvseries.match(name) is not None:
				searchString = tvseries.match(name).groups()[0]
			elif movietitle.match(name) is not None:
				searchString = movietitle.match(name).groups()[0]				
			else:
				searchString = name
			
			searchString = searchString.rstrip()
			
			filename = searchString.replace(".","_").replace(" ","_").replace("/","_")
			searchString = searchString.replace("."," ").replace("_"," ").replace("/"," ")
			tsname = serviceref.getPath()

			ptr = None
			
			jpg = "%s/%s.jpg" %(config.plugins.externalmovieinfoloader.coverpath.value, filename)
			
			if not fileExists(jpg):
				jpg = resolveFilename(SCOPE_CURRENT_SKIN, "cover_default.png")
				if not fileExists(jpg):
					jpg = None
					
			if jpg is not None:		
				ptr = LoadPixmap(cached=True, path=jpg)
					
			fileextension = serviceref.toString().split('.')
			fileextension = fileextension[-1].lower()				
			if not fileextension in ("mp3","mp2","m4a","wav","wma","flac"):
				self.movieList.append([serviceref, name, searchString, filename, tsname, ptr])
		
		self["list"].setList(self.movieList)


	# read all existing movies and add them to a list
	def getMovies(self):
		self.serviceHandler = eServiceCenter.getInstance()
		
		self.root = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + defaultMoviePath())
		
		self.movieList = []
		
		dirList = [self.root]
		
		for dir in dirList:
			fileList = self.serviceHandler.list(dir)
			while 1:
				serviceref = fileList.getNext()
				if not serviceref.valid():
					break
				
				info = self.serviceHandler.info(serviceref)
				if info:
					name = info.getName(serviceref)
				if serviceref.flags & eServiceReference.mustDescent:
					folderName = info.getName(serviceref)
					if folderName != "..":
						dirList.append(serviceref)
				else:
					if tvseriestotal.match(name) is not None:
						searchString = tvseriestotal.match(name).groups()[0]
					elif tvseriesalt.match(name) is not None:
						searchString = tvseriesalt.match(name).groups()[0]
					elif tvseries.match(name) is not None:
						searchString = tvseries.match(name).groups()[0]
					elif movietitle.match(name) is not None:
						searchString = movietitle.match(name).groups()[0]				
					else:
						searchString = name
			
					searchString = searchString.rstrip()
			
					filename = searchString.replace(".","_").replace(" ","_").replace("/","_")
					searchString = searchString.replace("."," ").replace("_"," ").replace("/"," ")
					tsname = serviceref.getPath()

					ptr = None
			
					jpg = "%s/%s.jpg" %(config.plugins.externalmovieinfoloader.coverpath.value, filename)
			
					if not fileExists(jpg):
						jpg = resolveFilename(SCOPE_CURRENT_SKIN, "cover_default.png")
						if not fileExists(jpg):
							jpg = None
					
					if jpg is not None:		
						ptr = LoadPixmap(cached=True, path=jpg)
					
					fileextension = serviceref.toString().split('.')
					fileextension = fileextension[-1].lower()				
					if not fileextension in ("mp3","mp2","m4a","wav","wma","flac"):
						self.movieList.append([serviceref, name, searchString, filename, tsname, ptr])
		
		# sort moiveList by searchString
		self.movieList.sort(key=lambda x: x[2].lower())
		self["list"].setList(self.movieList)

	# get data for all movies in the list
	def getMovieDataForAll(self):
		self.index = 0
		self.folderSearch = True
		self.descriptionOnly = False
		
		self.getMovieData()

	# get data for a single movie		
	def getMovieDataForSelected(self):
		self.index = self["list"].getIndex()
		self.folderSearch = False
		self.descriptionOnly = False
		
		# only load description / no need to check existence of cover
		if config.plugins.externalmovieinfoloader.mode.value == "description":
			self.getMovieData()
		else:
			# check existence of cover
			if fileExists("%s/%s.jpg" %(config.plugins.externalmovieinfoloader.coverpath.value, self.movieList[self.index][3])):
				# check whether to ask
				if config.plugins.externalmovieinfoloader.askforreplace.value:
					self.session.openWithCallback(self.processAnswer, MessageBox,_("The cover %s already exists. Do you want to replace it?") %(self.movieList[self.index][3]), MessageBox.TYPE_YESNO, title="External Movie Info Loader")
				else:
					# don't ask
					self.getMovieData()
			# cover does not exist
			else:				
				self.getMovieData()

	def processAnswer(self, answer):
		# delete cover
		if answer:
			os_system("rm %s/%s.jpg" %(config.plugins.externalmovieinfoloader.coverpath.value, self.movieList[self.index][3]))
			self.getMovieData()
		# do not delete cover
		else:
			# only continue when description shall be loaded
			if config.plugins.externalmovieinfoloader.mode.value != "cover":
				self.descriptionOnly = True
				self.getMovieData()
			
	def getMovieData(self):
		if self.tmdbconfig == False:
			self.getTMDBConfig()
		else:
			self.getTMDBData(self.movieList[self.index][2])
			
	#get data from TMDB for the selected movie
	def getTMDBData(self, searchString=""):
		headers = {
			'Accept':'application/json',
		}
		
		self.searchString = searchString
		
		getPage('%ssearch/%s?api_key=%s&query=%s&language=%s' %(BASEURL, self.movietype, APIKEY, quote(self.searchString), config.plugins.externalmovieinfoloader.infolanguage.value), method='GET', headers=headers).addCallback(self.searchResult).addErrback(self.getError)

	# parse the result of the tmdb-call, then open a screen to display the result	
	def searchResult(self, result):
		self.result_dict = json.loads(result)
		
		#print "result", self.result_dict
		
		totalresults = 0
		totalresults = self.result_dict['total_results']
		totalpages = 0
		totalpages = self.result_dict['total_pages']
		self.menulist = []
		
		movie_id = ""
		poster = ""
		name = ""

		if self.folderSearch and config.plugins.externalmovieinfoloader.usefirstentry.value:
			if totalresults > 0:
				maxvalue = 1
			else:
				self.closeOrSearch()
		else:			
			for p in range(0, totalpages):
				if p == totalpages-1:
					totalresultsreal = len(self.result_dict['results'])
					if totalresultsreal < totalresults:
						maxvalue = totalresultsreal%20
					else:
						maxvalue = totalresults%20
				else:
					maxvalue = 20
		
		self.buildMenuList(maxvalue)	
			
		self.session.openWithCallback(self.closeOrSearch,ExternalMovieInfoLoaderSearchResult, self.menulist, self.searchString, self.folderSearch, self.descriptionOnly)

	# build a menulist with the search result			
	def buildMenuList(self, maxvalue):
		type = self.movietype
	
		for i in range(0,maxvalue):
			poster = self.result_dict['results'][i]['poster_path']
			movie_id = self.result_dict['results'][i]['id']
			if self.movietype == 'multi':
				type = self.result_dict['results'][i]['media_type']
			if type == "tv":
				movietype = _("TV series")
				name = self.result_dict['results'][i]['name']
				airdate = self.result_dict['results'][i]['first_air_date']
			elif type == "movie":
				movietype = _("movie")
				name = self.result_dict['results'][i]['title']
				airdate = self.result_dict['results'][i]['release_date']
				
			url = str(self.picurl) + str(self.picsize) + str(self.result_dict['results'][i]['poster_path'])
			
			self.menulist.append((str(name), str(airdate), str(self.result_dict['results'][i]['poster_path']), str(self.result_dict['results'][i]['id']), url, self.movieList[self.index][3], movietype, self.movieList[self.index][4]))

	#get config mainly to get the available sizes and the url for the posters		
	def getTMDBConfig(self):
		headers = {
			'Accept':'application/json',
		}	
		getPage('%sconfiguration?api_key=%s' %(BASEURL, APIKEY), method='GET', headers = headers).addCallback(self.configResult).addErrback(self.getError)
	
	# store values from config-call
	def configResult(self, result):
		config_dict = json.loads(result)
		
		self.picurl = config_dict['images']['base_url']
		
		sizes = config_dict['images']['poster_sizes']
		for s in sizes:
			if s == 'w185':
				self.picsize = s
				break
			if s == 'w342':
				self.picsize = s
				break
				
		self.tmdbconfig = True
		self.getTMDBData(self.movieList[self.index][2])

	# in case of an error print it			
	def getError(self, error):
		print error		

	def downloadCover(self):
		self.filename = ""
		if len(self.menulist) > 0:
			if self.folderSearch and config.plugins.externalmovieinfoloader.usefirstentry.value:
				selectedIndex = 0
			else:
				selectedIndex = self["list"].index
	
			url = self.menulist[selectedIndex][4]
			path = config.plugins.externalmovieinfoloader.coverpath.value
			self.filename = self.menulist[selectedIndex][5]
	
			if not fileExists(path):
				os_system("mkdir %s" %(path))
		
			downloadPage(url, file("%s/%s.jpg" %(path, self.filename),'w')).addErrback(self.getError).addCallback(self.downloadFinished)
		
	def downloadFinished(self, result):
		print "[EMIL] - finished download of %s" %(self.filename)
		if self.folderSearch and config.plugins.externalmovieinfoloader.usefirstentry.value:
			pass
		else:
			self.session.open(MessageBox,_("The cover %s was downloaded successfully") %(self.filename), MessageBox.TYPE_INFO, timeout=5, title=_("External Movie Info Loader"))

	def exitDialog(self):
		self.close()
		if self.parent is not None:
			# This will try to get back to an updated movie list.
			# A proper way to do this should be provided in enigma2.
			try:
				parent = self.parent
				parent.csel.reloadList()
				parent.close()
			except AttributeError:
				try:
					# when started from MovieSelection Quickbutton Plugin, MovieSelection is parent, not MovieContextMenu --> try again
					self.parent.reloadList()
				except: pass		

	def closeOrSearch(self, searchString=""):
		if searchString == "":
			if self.folderSearch:
				if self.index < len(self.movieList)-1:
					self.index += 1
					self.getMovieData()
		# searchString is only set when it was edited
		elif searchString == "exitDialog":
			self.close()
		else:
			self.getTMDBData(searchString)

def context(session, service, **kwargs):
	session.open(ExternalMovieInfoLoader, service, session.current_dialog)
		
def main(session, **kwargs):
	session.open(ExternalMovieInfoLoader)

def Plugins(**kwargs):
	list = [PluginDescriptor(name=_("External Movie Info Loader (EMIL)"), description=_("Plugin to download external movie info"), where = [PluginDescriptor.WHERE_PLUGINMENU], fnc=main),PluginDescriptor(name=_("External Movie Info Loader (EMIL)"), description= _("External Movie Info Loader (EMIL)"), where = [PluginDescriptor.WHERE_MOVIELIST], fnc=context)]
	return list
